# The name of the install
Name "WPF ShaderEffect Generator"

# General Symbol Definitions
!define REGKEY "SOFTWARE\$(^Name)"
!define PRODUCT "WPF ShaderEffect Generator"
!define VERSION "1.7.1"
!define NAMEVERSION "WPF ShaderEffect Generator 1.7.1"
!define COMPANY "Nick Darnell"
!define URL "http://www.nickdarnell.com"


#####################################################################################
## UI Settings
#####################################################################################

!define MUI_FINISHPAGE_NOAUTOCLOSE
!define MUI_UNFINISHPAGE_NOAUTOCLOSE

; MUI Settings / Icons
!define MUI_ICON "${NSISDIR}\Contrib\Graphics\Icons\orange-install.ico"
!define MUI_UNICON "${NSISDIR}\Contrib\Graphics\Icons\orange-uninstall.ico"

; MUI Settings / Header
!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_RIGHT
!define MUI_HEADERIMAGE_BITMAP "rightimage.bmp"
!define MUI_HEADERIMAGE_UNBITMAP "rightimage.bmp"
 
; MUI Settings / Wizard
#Bitmap for the Welcome page and the Finish page (recommended size: 164x314 pixels).
!define MUI_WELCOMEFINISHPAGE_BITMAP "welcome_finish.bmp"
!define MUI_UNWELCOMEFINISHPAGE_BITMAP "welcome_finish.bmp"

# This will remove the description box on the component page where we list
# all the stuff we're planning to install.
!define MUI_COMPONENTSPAGE_SMALLDESC

#####################################################################################
## Includes
#####################################################################################

!include LogicLib.nsh
!include Sections.nsh
!include MUI2.nsh

# Include installer utility functions
!include "Include\InstallerUtil.nsh"


#####################################################################################
## UI Pages
#####################################################################################

# Installer pages
!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_COMPONENTS
!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_PAGE_FINISH
!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES

# Installer languages
!insertmacro MUI_LANGUAGE English

# Installer attributes
OutFile "${NAMEVERSION}.exe"
CRCCheck on
XPStyle on
ShowInstDetails show
VIProductVersion 1.7.1.0
VIAddVersionKey ProductName "${PRODUCT}"
VIAddVersionKey ProductVersion "${VERSION}"
VIAddVersionKey CompanyName "${COMPANY}"
VIAddVersionKey CompanyWebsite "${URL}"
VIAddVersionKey FileVersion "${VERSION}"
VIAddVersionKey FileDescription "${PRODUCT}"
VIAddVersionKey LegalCopyright "Nick Darnell 2010"
InstallDirRegKey HKLM "${REGKEY}" Path
ShowUninstDetails show


#####################################################################################
## Visual Studio 2008 Install Section
#####################################################################################

# Installer sections
Section /o "Visual Studio 2008" VS2008Sec
    SetOverwrite on


	SetOutPath "$INSTDIR\9.0\"
    File "..\Source\WPFShaderEffectGenerator\bin\VC90\Release\WPFShaderEffectGenerator.dll"
    File "..\Source\WPFShaderEffectGenerator\bin\VC90\Release\dx9fxutil.dll"
    File "Files\d3dx9_43.dll"
    File "Files\d3dx10_43.dll"
    File "Files\d3dx11_43.dll"
    File "Files\D3DCompiler_43.dll"
	

	SetOutPath "$DOCUMENTS\Visual Studio 2008\Templates\ItemTemplates\Visual C#\"
	File "..\Source\WPFShaderItemTemplate.zip"
	SetOutPath "$DOCUMENTS\Visual Studio 2008\Templates\ItemTemplates\Visual C#\WPF\"
	File "..\Source\WPFShaderItemTemplate.zip"
    SetOutPath "$DOCUMENTS\Visual Studio 2008\Templates\ItemTemplates\Visual C#\Silverlight\"
	File "..\Source\SLShaderItemTemplate.zip"


    # Register the package
	!include "Include\RegisterVC90.nsh"

SectionEnd

#####################################################################################
## Visual Studio 2010 Install Section
#####################################################################################

Section /o "Visual Studio 2010" VS2010Sec
    SetOverwrite on


	ExpandEnvStrings $0 "%VS100COMNTOOLS%..\IDE\Extensions\WPFShaderEffectGenerator\"
	SetOutPath "$0"
    File /r "..\Source\WPFShaderEffectGenerator\bin\VC100\Release\"
    File "Files\d3dx9_43.dll"
    File "Files\d3dx10_43.dll"
    File "Files\d3dx11_43.dll"
    File "Files\D3DCompiler_43.dll"

SectionEnd

Section -InstallFinal InstallFinalSec

    WriteRegStr HKLM "${REGKEY}\Components" Main 1

    WriteRegStr HKLM "${REGKEY}" Path $INSTDIR
    SetOutPath $INSTDIR
    WriteUninstaller $INSTDIR\uninstall.exe
    WriteRegStr HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" DisplayName "$(^Name)"
    WriteRegStr HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" DisplayVersion "${VERSION}"
    WriteRegStr HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" Publisher "${COMPANY}"
    WriteRegStr HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" URLInfoAbout "${URL}"
    WriteRegStr HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" DisplayIcon $INSTDIR\uninstall.exe
    WriteRegStr HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" UninstallString $INSTDIR\uninstall.exe
    WriteRegDWORD HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" NoModify 1
    WriteRegDWORD HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)" NoRepair 1
	
SectionEnd

#####################################################################################
## Installer Section Descriptions
#####################################################################################

LangString DESC_VS2008Sec ${LANG_ENGLISH} "The Visual Studio 2008 integration."
LangString DESC_VS2010Sec ${LANG_ENGLISH} "The Visual Studio 2010 integration."

!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${VS2008Sec} $(DESC_VS2008Sec)
  !insertmacro MUI_DESCRIPTION_TEXT ${VS2010Sec} $(DESC_VS2010Sec)
!insertmacro MUI_FUNCTION_DESCRIPTION_END


#####################################################################################
## Uninstaller Macros
#####################################################################################

# Macro for selecting uninstaller sections
!macro SELECT_UNSECTION SECTION_NAME UNSECTION_ID
    Push $R0
    ReadRegStr $R0 HKLM "${REGKEY}\Components" "${SECTION_NAME}"
    StrCmp $R0 1 0 next${UNSECTION_ID}
    !insertmacro SelectSection "${UNSECTION_ID}"
    GoTo done${UNSECTION_ID}
next${UNSECTION_ID}:
    !insertmacro UnselectSection "${UNSECTION_ID}"
done${UNSECTION_ID}:
    Pop $R0
!macroend

#####################################################################################
## Uninstaller Sections
#####################################################################################

# Uninstaller sections
Section /o -un.Main UNSEC0000

    #Get the "My Documents" folder and store it in $1
	System::Call 'shell32::SHGetSpecialFolderPathA(i $HWNDPARENT, t .r1, i ${CSIDL_DOCUMENTS}, b 'false') i r0'
 
    # Unregister the VS2008 Package
	!include "Include\UnregisterVC90.nsh"

    # Delete VS2008 Installed Files
    RmDir /r /REBOOTOK $INSTDIR

	# Remove VS2008 item templates
	Delete "$1\Visual Studio 2008\Templates\ItemTemplates\Visual C#\WPFShaderItemTemplate.zip"
	Delete "$1\Visual Studio 2008\Templates\ItemTemplates\Visual C#\WPF\WPFShaderItemTemplate.zip"
	
	# Remove VS2010 extension files
	ExpandEnvStrings $0 "%VS100COMNTOOLS%..\IDE\Extensions\WPFShaderEffectGenerator\"
	SetOutPath "$0"
    RmDir /r /REBOOTOK "$0"

    DeleteRegValue HKLM "${REGKEY}\Components" Main
SectionEnd

Section -un.post UNSEC0001

    # Delete registry information
    DeleteRegKey HKLM "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\$(^Name)"
    Delete /REBOOTOK $INSTDIR\uninstall.exe
    DeleteRegValue HKLM "${REGKEY}" Path
    DeleteRegKey /IfEmpty HKLM "${REGKEY}\Components"
    DeleteRegKey /IfEmpty HKLM "${REGKEY}"

    RmDir /REBOOTOK $INSTDIR
    
SectionEnd

#####################################################################################
## Installer Functions
#####################################################################################


# Installer functions
Function .onInit
	InitPluginsDir
	
	FindProcDLL::FindProc "devenv.exe"
	${If} $R0 == 1       
        MessageBox MB_OK|MB_ICONINFORMATION "Please exit all instances of Visual Studio before installing." IDCANCEL
		Quit
    ${EndIf}
    
    Call UninstallPrevious
	
	StrCpy $INSTDIR "$PROGRAMFILES\${PRODUCT}\"
	
		# Check if Visual Studio 2008 is installed
	ClearErrors
    ReadRegStr $0 HKCR "VisualStudio.DTE.9.0\" ""
    ${IfNot} ${Errors}
        SectionGetFlags "${VS2008Sec}" $0
        IntOp $0 $0 | ${SF_SELECTED}
        SectionSetFlags "${VS2008Sec}" $0
    ${EndIf}
	
	# Check if Visual Studio 2010 is installed
	ClearErrors
    ReadRegStr $0 HKCR "VisualStudio.DTE.10.0\" ""
	${IfNot} ${Errors}
        SectionGetFlags "${VS2010Sec}" $0
        IntOp $0 $0 | ${SF_SELECTED}
        SectionSetFlags "${VS2010Sec}" $0
    ${EndIf}
	
	ClearErrors
	
FunctionEnd

#####################################################################################
## Uninstaller Functions
#####################################################################################

# Uninstaller functions
Function un.onInit

	FindProcDLL::FindProc "devenv.exe"
	${If} $R0 == 1       
        MessageBox MB_OK|MB_ICONINFORMATION "Please exit all instances of Visual Studio before installing." IDCANCEL
		Quit
    ${EndIf}
	
	
    ReadRegStr $INSTDIR HKLM "${REGKEY}" Path
    !insertmacro SELECT_UNSECTION Main ${UNSEC0000}
FunctionEnd

# Checks for a previous version of the installer.  If it finds one, the user is given
# the option to uninstall it.  If they cancel, we abort the install.
Function UninstallPrevious
    # Check for uninstaller.
    ReadRegStr $R0 HKLM "${REGKEY}" "Path"
    ClearErrors
    ${If} $R0 == ""        
        Goto done
    ${EndIf}
    
    # We check if the uninstaller actually exists.  If it does, we can run the uninstall.  If it doesn't
    # there's nothing we can really do, so just perform the install.
    IfFileExists "$R0\uninstall.exe" uninstall 
    Goto done
    
uninstall:
    MessageBox MB_OKCANCEL|MB_ICONINFORMATION "A previous version was found, we need to uninstall the old version first" IDCANCEL exit

    # This will run the uninstaller 
    Exec '"$R0\uninstall.exe"'
    
exit:
    Quit
    
done:

FunctionEnd
